"use strict";
require("es6-promise").polyfill();
try {
    // optional
    require("source-map-support").install();
}
catch (e) {
}
var updateNotifier = require("update-notifier");
var pkg = require("../package.json");
var notifier = updateNotifier({
    packageName: pkg.name,
    packageVersion: pkg.version
});
if (notifier.update) {
    notifier.notify();
}
var dtsm = require("./");
var is = require("./incrementalsearch");
var commandpost = require("commandpost");
var archy = require("archy");
var root = commandpost
    .create("dtsm")
    .version(pkg.version, "-v, --version")
    .option("--insight <use>", "send usage opt in/out. in = `--insight true`, out = `--insight false`")
    .option("--offline", "offline first")
    .option("--remote <uri>", "uri of remote repository")
    .option("--config <path>", "path to json file")
    .option("--ref <ref>", "ref of repository")
    .action(function () {
    process.stdout.write(root.helpText() + '\n');
});
root
    .subCommand("init")
    .description("make new dtsm.json")
    .action(function () {
    setup(root.parsedOpts)
        .then(function (manager) {
        var jsonContent = manager.init();
        console.log("write to " + manager.configPath);
        console.log(jsonContent);
    })
        .catch(errorHandler);
});
root
    .subCommand("search [phrase]")
    .description("search .d.ts files")
    .option("--raw", "output search result by raw format")
    .option("-i, --interactive", "do incremental searching. use `peco` default")
    .action(function (opts, args) {
    setup(root.parsedOpts)
        .then(function (manager) {
        return manager.search(args.phrase || "");
    })
        .then(function (resultList) {
        if (!opts.interactive) {
            return Promise.resolve(resultList);
        }
        var candidates = resultList.map(function (result) { return result.fileInfo.path; });
        if (candidates.length === 0) {
            return Promise.resolve(resultList);
        }
        return is
            .exec(candidates, [args.phrase], "peco")
            .then(function (selected) {
            return selected
                .map(function (path) {
                return resultList.filter(function (result) { return result.fileInfo.path === path; })[0];
            })
                .filter(function (result) { return !!result; });
        });
    })
        .then(function (resultList) {
        if (opts.raw) {
            resultList.forEach(function (result) {
                console.log(result.fileInfo.path);
            });
        }
        else {
            if (resultList.length === 0) {
                console.log("No results.");
            }
            else {
                console.log("Search results.");
                console.log("");
                resultList.forEach(function (result) {
                    console.log("\t" + result.fileInfo.path);
                });
            }
        }
    })
        .catch(errorHandler);
});
root
    .subCommand("fetch")
    .description("fetch all data from remote repos")
    .action(function () {
    setup(root.parsedOpts)
        .then(function (manager) {
        process.stdout.write("fetching...\n");
        return manager.fetch();
    })
        .catch(errorHandler);
});
root
    .subCommand("install [files...]")
    .description("install .d.ts files")
    .option("-S, --save", "save .d.ts file path into dtsm.json")
    .option("--dry-run", "save .d.ts file path into dtsm.json")
    .option("-i, --interactive", "do incremental searching. use `peco` default")
    .action(function (opts, args) {
    // .action((...targets:string[])=> {
    setup(root.parsedOpts)
        .then(function (manager) {
        if (!opts.interactive && args.files.length === 0) {
            return manager.installFromFile({ dryRun: opts.dryRun })
                .then(function (result) { return printResult(result); })
                .then(function () { return manager.link({ dryRun: opts.dryRun }); })
                .then(function (result) { return printLinkResult(result); });
        }
        else if (opts.interactive || args.files.length === 0) {
            return manager.search("")
                .then(function (resultList) {
                var candidates = resultList.map(function (result) { return result.fileInfo.path; });
                if (candidates.length === 0) {
                    return Promise.resolve(resultList);
                }
                return is
                    .exec(candidates, args.files, "peco")
                    .then(function (selected) {
                    return selected
                        .map(function (path) {
                        return resultList.filter(function (result) { return result.fileInfo.path === path; })[0];
                    })
                        .filter(function (result) { return !!result; });
                });
            })
                .then(function (resultList) {
                var files = resultList.map(function (result) { return result.fileInfo.path; });
                return manager.install({ save: opts.save, dryRun: opts.dryRun }, files)
                    .then(function (result) { return printResult(result); });
            });
        }
        else {
            return manager.install({ save: opts.save, dryRun: opts.dryRun }, args.files)
                .then(function (result) { return printResult(result); });
        }
    })
        .catch(errorHandler);
});
root
    .subCommand("uninstall [files...]")
    .description("uninstall .d.ts files")
    .option("-S, --save", "save .d.ts file path into dtsm.json")
    .option("--dry-run", "save .d.ts file path into dtsm.json")
    .action(function (opts, args) {
    // .action((...targets:string[])=> {
    setup(root.parsedOpts)
        .then(function (manager) {
        return manager.uninstall({ save: opts.save, dryRun: opts.dryRun }, args.files);
    })
        .then(function (resultList) { return resultList.forEach(function (dep) { return console.log(dep.depName); }); })
        .catch(errorHandler);
});
root
    .subCommand("update")
    .description("update definition files version")
    .option("-S, --save", "save updated ref into dtsm.json")
    .option("--dry-run", "save .d.ts file path into dtsm.json")
    .action(function (opts, args) {
    setup(root.parsedOpts)
        .then(function (manager) {
        return manager.update({ save: opts.save, dryRun: opts.dryRun });
    })
        .then(function (result) { return printResult(result); })
        .catch(errorHandler);
});
root
    .subCommand("link")
    .description("link to npm or bower dependencies")
    .option("-S, --save", "save updated ref into dtsm.json")
    .option("--dry-run", "save .d.ts file path into dtsm.json")
    .action(function (opts, args) {
    setup(root.parsedOpts)
        .then(function (manager) {
        return manager.link({ save: opts.save, dryRun: opts.dryRun });
    })
        .then(function (result) { return printLinkResult(result); })
        .catch(errorHandler);
});
root
    .subCommand("refs")
    .description("show refs, it can use with --ref option")
    .action(function (opts, args) {
    setup(root.parsedOpts)
        .then(function (manager) { return manager.refs(); })
        .then(function (refs) {
        refs = refs.filter(function (ref) {
            // ignore github pull refs
            if (ref.name.indexOf("refs/pull/") === 0) {
                return false;
            }
            return true;
        });
        var branches = refs.filter(function (ref) { return ref.name.indexOf("refs/heads/") === 0; });
        var tags = refs.filter(function (ref) { return ref.name.indexOf("refs/tags/") === 0; });
        if (branches.length !== 0) {
            console.log("Branches:");
            branches.forEach(function (ref) {
                var branchName = ref.name.substr("refs/heads/".length);
                console.log("\t", branchName);
            });
            console.log("");
        }
        if (tags.length !== 0) {
            console.log("Tags:");
            tags.forEach(function (ref) {
                var tagName = ref.name.substr("refs/tags/".length);
                console.log("\t", tagName);
            });
            console.log("");
        }
    })
        .catch(errorHandler);
});
commandpost
    .exec(root, process.argv)
    .catch(errorHandler);
function setup(opts) {
    "use strict";
    var offline = opts.offline;
    var configPath = opts.config[0];
    var remoteUri = opts.remote[0];
    var specifiedRef = opts.ref[0];
    var insightStr = opts.insight[0];
    var insightOptout;
    if (typeof insightStr === "string") {
        if (insightStr !== "true" && insightStr !== "false") {
            return Promise.reject("--insight options required \"true\" or \"false\"");
        }
        else if (insightStr === "true") {
            insightOptout = false; // inverse
        }
        else {
            insightOptout = true; // inverse
        }
    }
    var repos = [];
    if (remoteUri || specifiedRef) {
        repos.push({
            url: remoteUri,
            ref: specifiedRef
        });
    }
    var options = {
        configPath: configPath || "dtsm.json",
        repos: repos,
        offline: offline,
        insightOptout: insightOptout
    };
    return dtsm
        .createManager(options)
        .then(function (manager) {
        return manager.tracker
            .askPermissionIfNeeded()
            .then(function () { return manager; });
    });
}
function printResolvedDependency(dep, opts) {
    "use strict";
    var fileInfo = function (dep) {
        if (!!dep.parent && dep.parent.repo === dep.repo && dep.parent.ref === dep.ref) {
            // emit only root node
            return "";
        }
        var result = "";
        if (opts.emitRepo && opts.emitHost) {
            if (dep.repoInstance.urlInfo) {
                result += dep.repoInstance.urlInfo.hostname;
            }
            else if (dep.repoInstance.sshInfo) {
                result += dep.repoInstance.sshInfo.hostname;
            }
        }
        if (opts.emitRepo) {
            var path;
            if (dep.repoInstance.urlInfo) {
                path = dep.repoInstance.urlInfo.pathname;
            }
            else if (dep.repoInstance.sshInfo) {
                path = dep.repoInstance.sshInfo.path;
            }
            var hasExt = /\.git$/.test(path);
            if (!opts.emitHost) {
                path = path.substr(1);
            }
            if (hasExt) {
                path = path.substr(0, path.length - 4);
            }
            result += path;
        }
        result += "#" + dep.fileInfo.ref.substr(0, 6);
        return " " + result;
    };
    var resultTree = function (dep, data) {
        var d = {
            label: dep.depName,
            nodes: []
        };
        if (!!data) {
            data.nodes.push(d);
        }
        d.label += fileInfo(dep);
        Object.keys(dep.dependencies).forEach(function (depName) {
            resultTree(dep.dependencies[depName], d);
        });
        if (!data) {
            return archy(d);
        }
        return null;
    };
    var output = resultTree(dep);
    console.log(output);
}
function printResult(result) {
    "use strict";
    // short is justice.
    var emitRepo = 1 < result.manager.repos.length;
    var emitHost = result.manager.repos.filter(function (repo) {
        if (!!repo.urlInfo) {
            return repo.urlInfo.host !== "github.com";
        }
        else if (!!repo.sshInfo) {
            return repo.sshInfo.hostname !== "github.com";
        }
        else {
            return true;
        }
    }).length !== 0;
    Object.keys(result.dependencies).forEach(function (depName) {
        var dep = result.dependencies[depName];
        printResolvedDependency(dep, { emitRepo: emitRepo, emitHost: emitHost });
    });
}
function printLinkResult(resultList) {
    "use strict";
    ["npm", "bower"].forEach(function (managerName) {
        var npmList = resultList.filter(function (result) { return result.managerName === managerName; });
        if (npmList.length !== 0) {
            var d = {
                label: "from " + managerName + " dependencies",
                nodes: npmList.map(function (dep) {
                    return {
                        label: dep.depName,
                        nodes: dep.files
                    };
                })
            };
            console.log(archy(d));
        }
    });
}
function errorHandler(err) {
    "use strict";
    if (err instanceof Error) {
        console.error(err.stack);
    }
    else {
        console.error(err);
    }
    return Promise.resolve(null).then(function () {
        process.exit(1);
    });
}
//# sourceMappingURL=cli.js.map