import * as fsgit from "fs-git";
import * as pmb from "packagemanager-backend";
import * as m from "./model";
import Tracker from "./tracker";
export declare function createManager(options?: m.Options): Promise<Manager>;
export default class Manager {
    static defaultConfigFile: string;
    static defaultRootDir: string;
    static defaultRepo: string;
    static defaultRef: string;
    static defaultPath: string;
    static defaultBundleFile: string;
    configPath: string;
    offline: boolean;
    rootDir: string;
    repos: pmb.RepositorySpec[];
    path: string;
    bundle: string;
    backend: pmb.Manager<m.GlobalConfig>;
    tracker: Tracker;
    savedRecipe: m.Recipe;
    constructor(options?: m.Options);
    _setupBackend(): Promise<Manager>;
    _setupDefaultRecipe(): void;
    init(path?: string): string;
    _load(path?: string): m.Recipe;
    _save(path?: string, recipe?: m.Recipe): string;
    search(phrase: string): Promise<pmb.SearchResult[]>;
    install(opts: {
        save: boolean;
        dryRun: boolean;
    }, phrases: string[]): Promise<pmb.Result>;
    installFromFile(opts?: {
        dryRun?: boolean;
    }, retryWithFetch?: boolean): Promise<pmb.Result>;
    _installFromOptions(recipe: m.Recipe, opts?: {
        dryRun?: boolean;
    }): Promise<pmb.Result>;
    _addWeightingAndSort<T>(phrase: string, list: T[], getPath: (val: T) => string): {
        weight: number;
        result: T;
    }[];
    update(opts: {
        save?: boolean;
        dryRun?: boolean;
    }): Promise<pmb.Result>;
    uninstall(opts: {
        save?: boolean;
        dryRun?: boolean;
    }, phrases: string[]): Promise<pmb.ResolvedDependency[]>;
    link(opts: {
        save?: boolean;
        dryRun?: boolean;
    }): Promise<m.LinkResult[]>;
    _processLink(linkInfo: m.Link, managerName: string, configFileName: string, moduleDir: string): {
        link: m.Link;
        results: m.LinkResult[];
    };
    _writeDefinitionFile(recipe: m.Recipe, depResult: pmb.ResolvedDependency): void;
    _removeDefinitionFile(recipe: m.Recipe, depResult: pmb.ResolvedDependency): void;
    _addReferenceToBundle(recipe: m.Recipe, pathFromCwd: string): void;
    refs(): Promise<fsgit.RefInfo[]>;
    fetch(): Promise<void>;
    _fetchIfOutdated(repo: pmb.Repo): Promise<pmb.Repo>;
    _fetchRepo(repo: pmb.Repo): Promise<pmb.Repo>;
    _checkOutdated(repoUrl: string): boolean;
    _getLastFetchAt(repoID: string): number;
    _setLastFetchAt(repoID: string, fetchAt?: number): void;
}
